@@ -14,12 +14,10 @@
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 import functools
 import hashlib
 import logging
+import math
 from collections import defaultdict
 from math import pi
 from pathlib import Path
@@ -53,9 +51,12 @@ from gajim.gtk.util import scale_with_ratio
 log = logging.getLogger('gajim.gtk.avatar')
 
 
-AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
+AvatarCacheT = dict[JID | str, dict[tuple[int, int, str | None, str | None],
                                           cairo.ImageSurface]]
 
+CIRCLE_RATIO = 0.18
+CIRCLE_FILL_RATIO = 0.80
+
 
 def generate_avatar_letter(text: str) -> str:
     return get_first_graphemes(text.lstrip(), 1).upper()
@@ -70,8 +71,7 @@ def generate_avatar(letters: str,
     color_r, color_g, color_b = color
 
     # Set up colors and size
-    if scale is not None:
-        size = size * scale
+    size = size * scale
 
     width = size
     height = size
@@ -134,6 +134,54 @@ def make_workspace_avatar(letter: str,
     return clip(surface, style)
 
 
+def add_transport_to_avatar(surface: cairo.ImageSurface,
+                            transport_icon: str,
+                            ) -> cairo.ImageSurface:
+
+    scale = surface.get_device_scale()[0]
+    width = surface.get_width()
+    height = surface.get_height()
+
+    transport_surface = load_icon_surface(
+        transport_icon,
+        math.floor(width / scale * CIRCLE_RATIO * 2),
+        int(scale))
+    if transport_surface is None:
+        return surface
+
+    new_surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+    new_surface.set_device_scale(*surface.get_device_scale())
+
+    context = cairo.Context(new_surface)
+    context.set_source_surface(surface, 0, 0)
+    context.paint()
+
+    width = width / scale
+
+    clip_radius = width * CIRCLE_RATIO
+    center_x = clip_radius
+    center_y = clip_radius
+
+    context.set_source_rgb(255, 255, 255)
+    context.set_operator(cairo.Operator.CLEAR)
+    context.arc(center_x, center_y, clip_radius, 0, 2 * pi)
+    context.fill()
+
+    clip_radius *= CIRCLE_FILL_RATIO
+    t_width = transport_surface.get_width() / scale
+    t_height = transport_surface.get_height() / scale
+
+    context.set_source_surface(
+        transport_surface,
+        center_x - t_width / 2,
+        center_y - t_height / 2)
+    context.set_operator(cairo.Operator.OVER)
+    context.arc(center_x, center_y, clip_radius, 0, 2 * pi)
+    context.fill()
+
+    return context.get_target()
+
+
 def add_status_to_avatar(surface: cairo.ImageSurface,
                          show: str
                          ) -> cairo.ImageSurface:
@@ -154,7 +202,7 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
     width = width / scale
     height = height / scale
 
-    clip_radius = width / 5.5
+    clip_radius = width * CIRCLE_RATIO
     center_x = width - clip_radius
     center_y = height - clip_radius
 
@@ -167,7 +215,7 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
     color = convert_rgb_string_to_float(
         app.css_config.get_value(css_color, StyleAttr.COLOR))
 
-    show_radius = clip_radius * 0.75
+    show_radius = clip_radius * CIRCLE_FILL_RATIO
 
     context.set_source_rgb(*color)
     context.set_operator(cairo.Operator.OVER)
@@ -187,7 +235,7 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
 
 
 @functools.lru_cache(maxsize=128)
-def get_show_circle(show: Union[str, types.PresenceShowT],
+def get_show_circle(show: str | types.PresenceShowT,
                     size: int,
                     scale: int
                     ) -> cairo.ImageSurface:
@@ -195,17 +243,20 @@ def get_show_circle(show: Union[str, types.PresenceShowT],
     if not isinstance(show, str):
         show = show.value
 
-    size = size * scale
-    center = size / 2
-    radius = size / 3
+    width = size * scale
+    height = width
 
-    surface = cairo.ImageSurface(cairo.Format.ARGB32, size, size)
+    surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+    surface.set_device_scale(scale, scale)
     context = cairo.Context(surface)
 
     css_color = get_css_show_class(show)
     color = convert_rgb_string_to_float(
         app.css_config.get_value(css_color, StyleAttr.COLOR))
 
+    center = size / 2
+    radius = size / 3
+
     context.set_source_rgb(*color)
     context.set_operator(cairo.Operator.OVER)
     context.arc(center, center, radius, 0, 2 * pi)
@@ -348,19 +399,19 @@ class AvatarStorage(metaclass=Singleton):
     def __init__(self):
         self._cache: AvatarCacheT = defaultdict(dict)
 
-    def invalidate_cache(self, jid: Union[JID, str]) -> None:
+    def invalidate_cache(self, jid: JID | str) -> None:
         self._cache.pop(jid, None)
 
     def get_pixbuf(self,
-                   contact: Union[types.BareContact,
-                                  types.GroupchatContact,
-                                  types.GroupchatParticipant],
+                   contact: (types.BareContact |
+                             types.GroupchatContact |
+                             types.GroupchatParticipant),
                    size: int,
                    scale: int,
-                   show: Optional[str] = None,
+                   show: str | None = None,
                    default: bool = False,
-                   transport_icon: Optional[str] = None,
-                   style: str = 'circle') -> Optional[GdkPixbuf.Pixbuf]:
+                   transport_icon: str | None = None,
+                   style: str = 'circle') -> GdkPixbuf.Pixbuf | None:
 
         surface = self.get_surface(
             contact, size, scale, show, default, transport_icon, style)
@@ -371,28 +422,20 @@ class AvatarStorage(metaclass=Singleton):
                                            size * scale)
 
     def get_surface(self,
-                    contact: Union[types.BareContact,
-                                   types.GroupchatContact,
-                                   types.GroupchatParticipant],
+                    contact: (types.BareContact |
+                              types.GroupchatContact |
+                              types.GroupchatParticipant),
                     size: int,
                     scale: int,
-                    show: Optional[str] = None,
+                    show: str | None = None,
                     default: bool = False,
-                    transport_icon: Optional[str] = None,
+                    transport_icon: str | None = None,
                     style: str = 'circle') -> cairo.ImageSurface:
 
         jid = contact.jid
 
-        if transport_icon is not None:
-            surface = load_icon_surface(transport_icon, size, scale)
-            assert surface is not None
-            if show is not None:
-                surface = add_status_to_avatar(surface, show)
-            self._cache[jid][(size, scale, show)] = surface
-            return surface
-
         if not default:
-            surface = self._cache[jid].get((size, scale, show))
+            surface = self._cache[jid].get((size, scale, show, transport_icon))
             if surface is not None:
                 return surface
 
@@ -400,7 +443,11 @@ class AvatarStorage(metaclass=Singleton):
             if surface is not None:
                 if show is not None:
                     surface = add_status_to_avatar(surface, show)
-                self._cache[jid][(size, scale, show)] = surface
+
+                if transport_icon is not None:
+                    surface = add_transport_to_avatar(surface, transport_icon)
+
+                self._cache[jid][(size, scale, show, transport_icon)] = surface
                 return surface
 
         name = contact.name
@@ -410,7 +457,11 @@ class AvatarStorage(metaclass=Singleton):
             letter, color, size, scale, style=style)
         if show is not None:
             surface = add_status_to_avatar(surface, show)
-        self._cache[jid][(size, scale, show)] = surface
+
+        if transport_icon is not None:
+            surface = add_transport_to_avatar(surface, transport_icon)
+
+        self._cache[jid][(size, scale, show, transport_icon)] = surface
         return surface
 
     def get_muc_surface(self,
@@ -419,31 +470,30 @@ class AvatarStorage(metaclass=Singleton):
                         size: int,
                         scale: int,
                         default: bool = False,
-                        transport_icon: Optional[str] = None,
-                        style: str = 'circle') -> Optional[cairo.ImageSurface]:
-
-        if transport_icon is not None:
-            surface = load_icon_surface(transport_icon, size, scale)
-            assert surface is not None
-            self._cache[jid][(size, scale, None)] = surface
-            return surface
+                        transport_icon: str | None = None,
+                        style: str = 'circle') -> cairo.ImageSurface:
 
         if not default:
-            surface = self._cache[jid].get((size, scale, None))
+            surface = self._cache[jid].get((size, scale, None, transport_icon))
             if surface is not None:
                 return surface
 
-            avatar_sha = app.storage.cache.get_muc(jid, 'avatar')
+            avatar_sha = app.storage.cache.get_muc(account, jid, 'avatar')
             if avatar_sha is not None:
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
+                    if transport_icon is not None:
+                        surface = add_transport_to_avatar(
+                            surface, transport_icon)
+
                     surface = clip(surface, style)
-                    self._cache[jid][(size, scale, None)] = surface
+                    self._cache[jid][
+                        (size, scale, None, transport_icon)] = surface
                     return surface
 
                 # avatar_sha set, but image is missing
                 # (e.g. avatar cache deleted)
-                app.storage.cache.set_muc(jid, 'avatar', None)
+                app.storage.cache.set_muc(account, jid, 'avatar', None)
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_bare_contact(jid)
@@ -452,15 +502,18 @@ class AvatarStorage(metaclass=Singleton):
         color = get_contact_color(contact)
         letter = generate_avatar_letter(name)
         surface = generate_default_avatar(letter, color, size, scale, style)
-        self._cache[jid][(size, scale, None)] = surface
+        if transport_icon is not None:
+            surface = add_transport_to_avatar(surface, transport_icon)
+
+        self._cache[jid][(size, scale, None, transport_icon)] = surface
         return surface
 
     def get_workspace_surface(self,
                               workspace_id: str,
                               size: int,
-                              scale: int) -> Optional[cairo.ImageSurface]:
+                              scale: int) -> cairo.ImageSurface | None:
 
-        surface = self._cache[workspace_id].get((size, scale, None))
+        surface = self._cache[workspace_id].get((size, scale, None, None))
         if surface is not None:
             return surface
 
@@ -480,11 +533,11 @@ class AvatarStorage(metaclass=Singleton):
         rgba = make_rgba(color or DEFAULT_WORKSPACE_COLOR)
         surface = make_workspace_avatar(
             name, rgba_to_float(rgba), size, scale)
-        self._cache[workspace_id][(size, scale, None)] = surface
+        self._cache[workspace_id][(size, scale, None, None)] = surface
         return surface
 
     @staticmethod
-    def _load_for_publish(path: str) -> Optional[tuple[bool, bytes]]:
+    def _load_for_publish(path: str) -> tuple[bool, bytes] | None:
         pixbuf = load_pixbuf(path)
         if pixbuf is None:
             return None
@@ -502,7 +555,7 @@ class AvatarStorage(metaclass=Singleton):
         return pixbuf.save_to_bufferv('png', [], [])
 
     @staticmethod
-    def save_avatar(data: bytes) -> Optional[str]:
+    def save_avatar(data: bytes) -> str | None:
         '''
         Save an avatar to the harddisk
 
@@ -510,8 +563,6 @@ class AvatarStorage(metaclass=Singleton):
 
         returns SHA1 value of the avatar or None on error
         '''
-        if data is None:
-            return None
 
         sha = hashlib.sha1(data).hexdigest()
         path = configpaths.get('AVATAR') / sha
@@ -519,12 +570,12 @@ class AvatarStorage(metaclass=Singleton):
             with open(path, 'wb') as output_file:
                 output_file.write(data)
         except Exception:
-            log.error('Storing avatar failed', exc_info=True)
+            log.exception('Storing avatar failed')
             return None
         return sha
 
     @staticmethod
-    def get_avatar_path(filename: str) -> Optional[Path]:
+    def get_avatar_path(filename: str) -> Path | None:
         path = configpaths.get('AVATAR') / filename
         if not path.is_file():
             return None
@@ -536,7 +587,7 @@ class AvatarStorage(metaclass=Singleton):
     def surface_from_filename(self,
                               filename: str,
                               size: int,
-                              scale: int) -> Optional[cairo.ImageSurface]:
+                              scale: int) -> cairo.ImageSurface | None:
 
         size = size * scale
         path = self.get_avatar_path(filename)
@@ -553,7 +604,7 @@ class AvatarStorage(metaclass=Singleton):
     def _load_surface_from_storage(self,
                                    filename: str,
                                    size: int,
-                                   scale: int) -> Optional[cairo.ImageSurface]:
+                                   scale: int) -> cairo.ImageSurface | None:
 
         size = size * scale
         path = self.get_avatar_path(filename)
@@ -567,12 +618,12 @@ class AvatarStorage(metaclass=Singleton):
         return fit(surface, size)
 
     def _get_avatar_from_storage(self,
-                                 contact: Union[types.BareContact,
-                                                types.GroupchatContact,
-                                                types.GroupchatParticipant],
+                                 contact: (types.BareContact |
+                                           types.GroupchatContact |
+                                           types.GroupchatParticipant),
                                  size: int,
                                  scale: int,
-                                 style: str) -> Optional[cairo.ImageSurface]:
+                                 style: str) -> cairo.ImageSurface | None:
 
         avatar_sha = contact.avatar_sha
         if avatar_sha is None:
