@@ -15,9 +15,9 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import cairo
+from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
 
@@ -26,19 +26,25 @@ from gajim.common import ged
 from gajim.common import types
 from gajim.common.const import AvatarSize
 from gajim.common.const import SimpleClientState
+from gajim.common.const import XmppUriQuery
 from gajim.common.events import AccountEnabled
 from gajim.common.events import BookmarksReceived
 from gajim.common.events import MessageReceived
-from gajim.common.events import MucDiscoUpdate
 from gajim.common.ged import EventHelper
+from gajim.common.helpers import generate_qr_code
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 
-from .builder import get_builder
-from .groupchat_voice_requests_button import VoiceRequestsButton
-from .util import AccountBadge
+from gajim.gtk.builder import get_builder
+from gajim.gtk.groupchat_voice_requests_button import VoiceRequestsButton
+from gajim.gtk.menus import get_groupchat_menu
+from gajim.gtk.menus import get_private_chat_menu
+from gajim.gtk.menus import get_self_contact_menu
+from gajim.gtk.menus import get_singlechat_menu
+from gajim.gtk.tooltips import ContactTooltip
+from gajim.gtk.util import AccountBadge
 
 
 class ChatBanner(Gtk.Box, EventHelper):
@@ -46,8 +52,8 @@ class ChatBanner(Gtk.Box, EventHelper):
         Gtk.Box.__init__(self)
         EventHelper.__init__(self)
 
-        self._client: Optional[types.Client] = None
-        self._contact: Optional[types.ChatContactT] = None
+        self._client: types.Client | None = None
+        self._contact: types.ChatContactT | None = None
 
         self._last_message_from_phone: set[BareContact] = set()
 
@@ -85,7 +91,6 @@ class ChatBanner(Gtk.Box, EventHelper):
         if not self.has_events_registered():
             self.register_events([
                 ('message-received', ged.GUI2, self._on_message_received),
-                ('muc-disco-update', ged.GUI2, self._on_muc_disco_update),
                 ('bookmarks-received', ged.GUI2, self._on_bookmarks_received),
                 ('account-enabled', ged.GUI2, self._on_account_changed),
                 ('account-disabled', ged.GUI2, self._on_account_changed)
@@ -93,12 +98,16 @@ class ChatBanner(Gtk.Box, EventHelper):
 
         self._voice_requests_button.switch_contact(self._contact)
 
+        self._set_chat_menu(contact)
         self._update_phone_image()
+        self._update_robot_image()
         self._update_roster_button()
         self._update_avatar()
         self._update_visitor_button()
         self._update_name_label()
+        self._update_description_label()
         self._update_account_badge()
+        self._update_share_box()
 
     def _connect_signals(self) -> None:
         assert self._contact is not None
@@ -116,7 +125,8 @@ class ChatBanner(Gtk.Box, EventHelper):
             self._contact.multi_connect({
                 'user-role-changed': self._on_user_role_changed,
                 'state-changed': self._on_muc_state_changed,
-                'room-voice-request': self._on_room_voice_request
+                'room-voice-request': self._on_room_voice_request,
+                'disco-info-update': self._on_disco_info_update,
             })
 
         elif isinstance(self._contact, GroupchatParticipant):
@@ -151,13 +161,16 @@ class ChatBanner(Gtk.Box, EventHelper):
                             ) -> None:
 
         self._update_avatar()
+        self._update_description_label()
 
     def _on_chatstate_update(self,
-                             _contact: types.BareContact,
+                             contact: types.BareContact,
                              _signal_name: str
                              ) -> None:
-
-        self._update_name_label()
+        if contact.is_groupchat:
+            self._update_description_label()
+        else:
+            self._update_name_label()
 
     def _on_nickname_update(self,
                             _contact: types.BareContact,
@@ -177,8 +190,8 @@ class ChatBanner(Gtk.Box, EventHelper):
                         _contact: types.BareContact,
                         _signal_name: str
                         ) -> None:
-
         self._update_avatar()
+        self._update_robot_image()
 
     def _on_muc_state_changed(self,
                               contact: GroupchatContact,
@@ -206,12 +219,13 @@ class ChatBanner(Gtk.Box, EventHelper):
 
         self._update_name_label()
 
-    def _on_muc_disco_update(self, event: MucDiscoUpdate) -> None:
-        assert self._contact is not None
-        if event.jid != self._contact.jid:
-            return
+    def _on_disco_info_update(self,
+                              _contact: GroupchatContact,
+                              _signal_name: str
+                              ) -> None:
 
         self._update_name_label()
+        self._update_description_label()
 
     def _on_account_changed(self, event: AccountEnabled) -> None:
         self._update_account_badge()
@@ -232,10 +246,28 @@ class ChatBanner(Gtk.Box, EventHelper):
 
         self._update_phone_image()
 
+    def _set_chat_menu(self, contact: types.ChatContactT) -> None:
+        if isinstance(contact, GroupchatContact):
+            menu = get_groupchat_menu(contact)
+        elif isinstance(contact, GroupchatParticipant):
+            menu = get_private_chat_menu(contact)
+        elif contact.is_self:
+            menu = get_self_contact_menu(contact)
+        else:
+            menu = get_singlechat_menu(contact)
+        self._ui.chat_menu_button.set_menu_model(menu)
+
     def _update_phone_image(self) -> None:
         self._ui.phone_image.set_visible(
             self._contact in self._last_message_from_phone)
 
+    def _update_robot_image(self) -> None:
+        if isinstance(self._contact, BareContact):
+            self._ui.robot_image.set_visible(self._contact.is_gateway
+                                             or self._contact.is_bot)
+        else:
+            self._ui.robot_image.set_visible(False)
+
     def _update_roster_button(self) -> None:
         self._ui.toggle_roster_button.set_visible(
             isinstance(self._contact, GroupchatContact))
@@ -247,6 +279,20 @@ class ChatBanner(Gtk.Box, EventHelper):
         assert isinstance(surface, cairo.ImageSurface)
         self._ui.avatar_image.set_from_surface(surface)
 
+        self._avatar_image_tooltip = ContactTooltip()
+
+    def _on_query_tooltip(self,
+                          _img: Gtk.Image,
+                          _x_coord: int,
+                          _y_coord: int,
+                          _keyboard_mode: bool,
+                          tooltip: Gtk.Tooltip) -> bool:
+        if not isinstance(self._contact, BareContact):
+            return False
+        res, widget = self._avatar_image_tooltip.get_tooltip(self._contact)
+        tooltip.set_custom(widget)
+        return res
+
     def _update_visitor_button(self) -> None:
         if not isinstance(self._contact, GroupchatContact):
             self._ui.visitor_box.set_visible(False)
@@ -283,6 +329,38 @@ class ChatBanner(Gtk.Box, EventHelper):
 
         self._ui.name_label.set_tooltip_text(tooltip_text)
 
+    def _get_muc_description_text(self) -> str:
+        contact = self._contact
+        assert isinstance(contact, GroupchatContact)
+
+        typing = contact.get_composers()
+        if not typing or not app.settings.get('show_chatstate_in_banner'):
+            disco_info = app.storage.cache.get_last_disco_info(contact.jid)
+            if disco_info is None:
+                return ''
+            return disco_info.muc_description or ''
+
+        composers = tuple(c.name for c in typing)
+        n = len(composers)
+        if n == 1:
+            return _('%s is typing…') % composers[0]
+        elif n == 2:
+            return _('%s and %s are typing…') % composers
+        else:
+            return _('%s participants are typing…') % n
+
+    def _update_description_label(self) -> None:
+        contact = self._contact
+        assert contact is not None
+
+        if contact.is_groupchat:
+            text = self._get_muc_description_text()
+        else:
+            assert not isinstance(contact, GroupchatContact)
+            text = contact.status or ''
+        self._ui.description_label.set_text(text)
+        self._ui.description_label.set_visible(bool(text))
+
     def _update_account_badge(self) -> None:
         if self._contact is None:
             self._account_badge.set_visible(False)
@@ -294,6 +372,52 @@ class ChatBanner(Gtk.Box, EventHelper):
 
         self._account_badge.set_visible(visible)
 
+    def _update_share_box(self) -> None:
+        assert self._contact is not None
+        if self._contact.is_groupchat:
+            self._ui.share_menu_button.set_tooltip_text(_('Share Group Chat…'))
+        else:
+            self._ui.share_menu_button.set_tooltip_text(_('Share Contact…'))
+        self._ui.share_menu_button.set_sensitive(
+            not self._contact.is_pm_contact)
+        self._ui.jid_label.set_text(str(self._contact.jid))
+
+    def _get_share_uri(self) -> str:
+        assert self._client is not None
+        assert self._contact is not None
+        jid = self._contact.get_address()
+        if self._contact.is_groupchat:
+            return jid.to_iri(XmppUriQuery.JOIN.value)
+        else:
+            return self._client.get_module('OMEMO').compose_trust_uri(jid)
+
+    def _on_share_clicked(self, _button: Gtk.Button) -> None:
+        assert self._contact is not None
+        if isinstance(self._contact, GroupchatContact):
+            share_text = _('Scan this QR code to join %s.')
+            if self._contact.muc_context == 'private':
+                share_text = _('%s can be joined by invite only.')
+        else:
+            share_text = _('Scan this QR code to start a chat with %s.')
+        self._ui.share_instructions.set_text(share_text % self._contact.name)
+
+        if (isinstance(self._contact, GroupchatContact) and
+                self._contact.muc_context == 'private'):
+            # Don't display QR code for private MUCs (they require an invite)
+            self._ui.qr_code_image.hide()
+            return
+
+        # Generate QR code on demand (i.e. not when switching chats)
+        self._ui.qr_code_image.set_from_pixbuf(
+            generate_qr_code(self._get_share_uri()))
+        self._ui.qr_code_image.show()
+
+    def _on_copy_jid_clicked(self, _button: Gtk.Button) -> None:
+        text = self._get_share_uri()
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clipboard.set_text(text, -1)
+        self._ui.share_popover.popdown()
+
     def _on_request_voice_clicked(self, _button: Gtk.Button) -> None:
         self._ui.visitor_popover.popdown()
         app.window.activate_action('muc-request-voice', None)
