@@ -15,9 +15,8 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Optional
 
+from collections.abc import Callable
 from datetime import datetime
 
 import cairo
@@ -32,10 +31,10 @@ from gajim.common.i18n import p_
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.types import ChatContactT
 
-from ...menus import get_groupchat_participant_menu
-from ...util import GajimPopover
-from ...util import get_cursor
-from ...util import wrap_with_event_box
+from gajim.gtk.menus import get_groupchat_participant_menu
+from gajim.gtk.util import GajimPopover
+from gajim.gtk.util import get_cursor
+from gajim.gtk.util import wrap_with_event_box
 
 
 class SimpleLabel(Gtk.Label):
@@ -74,6 +73,7 @@ class DateTimeLabel(Gtk.Label):
             format_string = app.settings.get('date_time_format')
 
         self.set_text(timestamp.strftime(format_string))
+        self.set_selectable(True)
         self.set_halign(Gtk.Align.START)
         self.set_valign(Gtk.Align.END)
         self.set_margin_start(6)
@@ -149,7 +149,7 @@ class AvatarBox(Gtk.EventBox):
     def __init__(self,
                  contact: ChatContactT,
                  name: str,
-                 avatar: Optional[cairo.ImageSurface],
+                 avatar: cairo.ImageSurface | None,
                  ) -> None:
 
         Gtk.EventBox.__init__(self)
@@ -168,7 +168,7 @@ class AvatarBox(Gtk.EventBox):
         self.connect('button-press-event',
                      self._on_avatar_clicked, name)
 
-    def set_from_surface(self, surface: Optional[cairo.ImageSurface]) -> None:
+    def set_from_surface(self, surface: cairo.ImageSurface | None) -> None:
         self._image.set_from_surface(surface)
 
     def set_merged(self, merged: bool) -> None:
@@ -191,9 +191,9 @@ class AvatarBox(Gtk.EventBox):
             if not isinstance(self._contact, GroupchatContact):
                 return Gdk.EVENT_STOP
 
-            if event.button == 1:
+            if event.button == Gdk.BUTTON_PRIMARY:
                 app.window.activate_action('mention', GLib.Variant('s', name))
-            elif event.button == 3:
+            elif event.button == Gdk.BUTTON_SECONDARY:
                 self._show_participant_menu(name, event)
 
         return Gdk.EVENT_STOP
