@@ -12,7 +12,6 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Optional
 
 from gi.repository import Gtk
 from nbxmpp.protocol import JID
@@ -20,6 +19,9 @@ from nbxmpp.protocol import JID
 from gajim.common import app
 from gajim.common import i18n
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.dialogs import InformationDialog
@@ -34,7 +36,7 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                  action: str,
                  exchange_list: dict[str, list[str]],
                  jid_from: JID,
-                 message_body: Optional[str] = None
+                 message_body: str | None = None
                  ) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
@@ -54,6 +56,8 @@ class RosterItemExchange(Gtk.ApplicationWindow):
 
         # Set label depending on action
         contact = self._client.get_module('Contacts').get_contact(jid_from)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         type_label = ''
         if action == 'add':
             type_label = _('%(name)s (%(jid)s) would like to add some '
@@ -127,6 +131,7 @@ class RosterItemExchange(Gtk.ApplicationWindow):
     def _add(self) -> None:
         for jid in self._exchange_list:
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
             groups = ''
             num_list = len(self._exchange_list[jid][1])
@@ -153,6 +158,7 @@ class RosterItemExchange(Gtk.ApplicationWindow):
         for jid in self._exchange_list:
             is_right = True
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
             groups = ''
 
@@ -183,6 +189,7 @@ class RosterItemExchange(Gtk.ApplicationWindow):
     def _delete(self) -> None:
         for jid in self._exchange_list:
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
             groups = ''
             num_list = len(self._exchange_list[jid][1])
@@ -217,11 +224,15 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                     # It is selected
                     contact = self._client.get_module('Contacts').get_contact(
                         self._jid_from)
+                    assert isinstance(
+                        contact,
+                        BareContact | GroupchatContact | GroupchatParticipant)
                     message = _('%(name)s %(jid)s suggested me to add you to '
                                 'my contact list.') % {
                                     'name': contact.name,
                                     'jid': self._jid_from}
                     # Keep same groups and same nickname
+                    groups: list[str] = []
                     groups = model[iter_][3].split(', ')
                     if groups == ['']:
                         groups = []
@@ -237,9 +248,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                         groups=groups,
                         auto_auth=True)
                 iter_ = model.iter_next(iter_)
-            InformationDialog(i18n.ngettext('Added %d contact',
-                                            'Added %d contacts',
-                                            count, str(count), str(count)))
+            InformationDialog(i18n.ngettext('Added %s contact',
+                                            'Added %s contacts',
+                                            count,
+                                            str(count),
+                                            str(count)))
         elif self._action == 'modify':
             count = 0
             while iter_:
@@ -265,9 +278,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                     self._client.get_module('Presence').unsubscribe(jid)
                     self._client.get_module('Roster').delete_item(jid)
                 iter_ = model.iter_next(iter_)
-            InformationDialog(i18n.ngettext('Removed %d contact',
-                                            'Removed %d contacts',
-                                            count, str(count), str(count)))
+            InformationDialog(i18n.ngettext('Removed %s contact',
+                                            'Removed %s contacts',
+                                            count,
+                                            str(count),
+                                            str(count)))
         self.destroy()
 
     def _on_cancel_button_clicked(self, _button: Gtk.Button) -> None:
